@@ -1,15 +1,39 @@
-/*
- * LibHTP (http://www.libhtp.org)
- * Copyright 2009,2010 Ivan Ristic <ivanr@webkreator.com>
- *
- * LibHTP is an open source product, released under terms of the General Public Licence
- * version 2 (GPLv2). Please refer to the file LICENSE, which contains the complete text
- * of the license.
- *
- * In addition, there is a special exception that allows LibHTP to be freely
- * used with any OSI-approved open source licence. Please refer to the file
- * LIBHTP_LICENSING_EXCEPTION for the full text of the exception.
- *
+/***************************************************************************
+ * Copyright (c) 2009-2010 Open Information Security Foundation
+ * Copyright (c) 2010-2013 Qualys, Inc.
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ * 
+ * - Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+
+ * - Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in the
+ *   documentation and/or other materials provided with the distribution.
+
+ * - Neither the name of the Qualys, Inc. nor the names of its
+ *   contributors may be used to endorse or promote products derived from
+ *   this software without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ ***************************************************************************/
+
+/**
+ * @file
+ * @author Ivan Ristic <ivanr@webkreator.com>
  */
 
 #include <fcntl.h>
@@ -26,7 +50,7 @@
 /**
  * Destroys a test.
  *
- * @param test
+ * @param[in] test
  */
 static void test_destroy(test_t *test) {
     if (test->buf != NULL) {
@@ -38,13 +62,13 @@ static void test_destroy(test_t *test) {
 /**
  * Checks if there's a chunk boundary at the given position.
  *
- * @param test
- * @param pos
+ * @param[in] test
+ * @param[in] pos
  * @return Zero if there is no boundary, SERVER or CLIENT if a boundary
  *         was found, and a negative value on error (e.g., not enough data
  *         to determine if a boundary is present).
  */
-static int test_is_boundary(test_t *test, int pos) {
+static int test_is_boundary(test_t *test, size_t pos) {
     // Check that there's enough room
     if (pos + 3 >= test->len) return -1;
 
@@ -80,11 +104,11 @@ static int test_is_boundary(test_t *test, int pos) {
 /**
  * Initializes test by loading the entire data file into a memory block.
  *
- * @param test
- * @param filename
+ * @param[in] test
+ * @param[in] filename
  * @return Non-negative value on success, negative value on error.
  */
-static int test_init(test_t *test, const char *filename) {
+static int test_init(test_t *test, const char *filename, int clone_count) {
     memset(test, 0, sizeof (test_t));
 
     int fd = open(filename, O_RDONLY | O_BINARY);
@@ -95,7 +119,7 @@ static int test_init(test_t *test, const char *filename) {
         return -1;
     }
 
-    test->buf = malloc(buf.st_size);
+    test->buf = malloc(buf.st_size * clone_count + clone_count - 1);
     test->len = 0;
     test->pos = 0;
 
@@ -111,17 +135,25 @@ static int test_init(test_t *test, const char *filename) {
 
     close(fd);
 
+    int i = 1;
+    for (i = 1; i < clone_count; i++) {
+        test->buf[i * buf.st_size + (i-1)] = '\n';
+        memcpy(test->buf + i * buf.st_size + i, test->buf, buf.st_size);
+    }
+    
+    test->len = buf.st_size * clone_count + clone_count - 1;
+
     return 1;
 }
 
-void test_start(test_t *test) {
+static void test_start(test_t *test) {
     test->pos = 0;
 }
 
 /**
  * Finds the next data chunk in the given test.
  *
- * @param test
+ * @param[in] test
  * @return One if a chunk is found or zero if there are no more chunks in the test. On
  *         success, test->chunk will point to the beginning of the chunk, while
  *         test->chunk_len will contain its length.
@@ -226,12 +258,12 @@ static int parse_filename(const char *filename, char **remote_addr, int *remote_
 /**
  * Runs a single test.
  *
- * @param filename
- * @param cfg
+ * @param[in] filename
+ * @param[in] cfg
  * @return A pointer to the instance of htp_connp_t created during
  *         the test, or NULL if the test failed for some reason.
  */
-int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_connp_t **connp) {
+int test_run_ex(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_connp_t **connp, int clone_count) {
     char filename[1025];
     test_t test;
     struct timeval tv_start, tv_end;
@@ -243,11 +275,11 @@ int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_con
     strncat(filename, "/", 1024 - strlen(filename));
     strncat(filename, testname, 1024 - strlen(filename));
 
-    printf("Filename: %s\n", filename);
+    // printf("Filename: %s\n", filename);
 
     // Initinialize test
 
-    rc = test_init(&test, filename);
+    rc = test_init(&test, filename, clone_count);
     if (rc < 0) {
         return rc;
     }
@@ -268,28 +300,28 @@ int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_con
     // Does the filename contain connection metdata?
     if (strncmp(testname, "stream", 6) == 0) {
         // It does; use it
-        char *remote_addr, *local_addr;
-        int remote_port, local_port;
+        char *remote_addr = NULL, *local_addr = NULL;
+        int remote_port = -1, local_port = -1;
 
         parse_filename(testname, &remote_addr, &remote_port, &local_addr, &local_port);
-        htp_connp_open(*connp, (const char *) remote_addr, remote_port, (const char *) local_addr, local_port, tv_start.tv_usec);
+        htp_connp_open(*connp, (const char *) remote_addr, remote_port, (const char *) local_addr, local_port, &tv_start);
         free(remote_addr);
         free(local_addr);
     } else {
         // No connection metadata; provide some fake information instead
-        htp_connp_open(*connp, (const char *) "127.0.0.1", 10000, (const char *) "127.0.0.1", 80, tv_start.tv_usec);
+        htp_connp_open(*connp, (const char *) "127.0.0.1", 10000, (const char *) "127.0.0.1", 80, &tv_start);
     }
 
     // Find all chunks and feed them to the parser
     int in_data_other = 0;
-    char *in_data;
-    size_t in_data_len;
-    size_t in_data_offset;
+    char *in_data = NULL;
+    size_t in_data_len = 0;
+    size_t in_data_offset = 0;
 
     int out_data_other = 0;
-    char *out_data;
-    size_t out_data_len;
-    size_t out_data_offset;
+    char *out_data = NULL;
+    size_t out_data_len = 0;
+    size_t out_data_offset = 0;
 
     for (;;) {
         if (test_next_chunk(&test) <= 0) {
@@ -302,13 +334,13 @@ int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_con
                 fprintf(stderr, "Unable to buffer more than one inbound chunk.\n");
                 return -1;
             }
-
-            int rc = htp_connp_req_data(*connp, tv_start.tv_usec, test.chunk, test.chunk_len);
-            if (rc == STREAM_STATE_ERROR) {
+            
+            rc = htp_connp_req_data(*connp, &tv_start, test.chunk, test.chunk_len);
+            if (rc == HTP_STREAM_ERROR) {
                 test_destroy(&test);
                 return -101;
             }
-            if (rc == STREAM_STATE_DATA_OTHER) {
+            if (rc == HTP_STREAM_DATA_OTHER) {
                 // Parser needs to see the outbound stream in order to continue
                 // parsing the inbound stream.
                 in_data_other = 1;
@@ -318,20 +350,21 @@ int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_con
             }
         } else {
             if (out_data_other) {
-                int rc = htp_connp_res_data(*connp, tv_start.tv_usec, out_data + out_data_offset, out_data_len - out_data_offset);
-                if (rc == STREAM_STATE_ERROR) {
+                rc = htp_connp_res_data(*connp, &tv_start, out_data + out_data_offset, out_data_len - out_data_offset);
+                if (rc == HTP_STREAM_ERROR) {
                     test_destroy(&test);
                     return -104;
                 }
+                
                 out_data_other = 0;
             }
 
-            int rc = htp_connp_res_data(*connp, tv_start.tv_usec, test.chunk, test.chunk_len);
-            if (rc == STREAM_STATE_ERROR) {
+            rc = htp_connp_res_data(*connp, &tv_start, test.chunk, test.chunk_len);
+            if (rc == HTP_STREAM_ERROR) {
                 test_destroy(&test);
                 return -102;
             }
-            if (rc == STREAM_STATE_DATA_OTHER) {
+            if (rc == HTP_STREAM_DATA_OTHER) {
                 // Parser needs to see the outbound stream in order to continue
                 // parsing the inbound stream.
                 out_data_other = 1;
@@ -342,19 +375,20 @@ int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_con
             }
 
             if (in_data_other) {
-                int rc = htp_connp_req_data(*connp, tv_start.tv_usec, in_data + in_data_offset, in_data_len - in_data_offset);
-                if (rc == STREAM_STATE_ERROR) {
+                rc = htp_connp_req_data(*connp, &tv_start, in_data + in_data_offset, in_data_len - in_data_offset);
+                if (rc == HTP_STREAM_ERROR) {
                     test_destroy(&test);
                     return -103;
                 }
+                
                 in_data_other = 0;
             }
         }
     }
 
     if (out_data_other) {
-        int rc = htp_connp_res_data(*connp, tv_start.tv_usec, out_data + out_data_offset, out_data_len - out_data_offset);
-        if (rc == STREAM_STATE_ERROR) {
+        rc = htp_connp_res_data(*connp, &tv_start, out_data + out_data_offset, out_data_len - out_data_offset);
+        if (rc == HTP_STREAM_ERROR) {
             test_destroy(&test);
             return -104;
         }
@@ -364,12 +398,14 @@ int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_con
     gettimeofday(&tv_end, NULL);
 
     // Close the connection
-    htp_connp_close(*connp, tv_end.tv_usec);
-
-    // printf("Parsing time: %i\n", tv_end.tv_usec - tv_start.tv_usec);
+    htp_connp_close(*connp, &tv_end);
 
     // Clean up
     test_destroy(&test);
 
     return 1;
 }
+
+int test_run(const char *testsdir, const char *testname, htp_cfg_t *cfg, htp_connp_t **connp) {
+    return test_run_ex(testsdir, testname, cfg, connp, 1);
+}
