@@ -28,6 +28,7 @@
 
 #include "app-layer-protos.h"
 #include "app-layer-parser.h"
+#include "app-layer.h"
 
 #include "util-spm.h"
 #include "util-unittest.h"
@@ -45,14 +46,16 @@ enum {
 };
 
 static uint32_t FragmentDataParser(Flow *f, void *dcerpcudp_state,
-		AppLayerParserState *pstate, uint8_t *input, uint32_t input_len,
-		AppLayerParserResult *output) {
+                                   AppLayerParserState *pstate,
+                                   uint8_t *input, uint32_t input_len)
+{
 	SCEnter();
 	DCERPCUDPState *sstate = (DCERPCUDPState *) dcerpcudp_state;
     uint8_t **stub_data_buffer = NULL;
     uint32_t *stub_data_buffer_len = NULL;
     uint8_t *stub_data_fresh = NULL;
     uint16_t stub_len = 0;
+    void *ptmp;
 
     /* request PDU.  Retrieve the request stub buffer */
     if (sstate->dcerpc.dcerpchdrudp.type == REQUEST) {
@@ -80,11 +83,15 @@ static uint32_t FragmentDataParser(Flow *f, void *dcerpcudp_state,
         *stub_data_buffer_len = 0;
     }
 
-    *stub_data_buffer = SCRealloc(*stub_data_buffer, *stub_data_buffer_len + stub_len);
-    if (*stub_data_buffer == NULL) {
+    ptmp = SCRealloc(*stub_data_buffer, *stub_data_buffer_len + stub_len);
+    if (ptmp == NULL) {
+        SCFree(*stub_data_buffer);
+        *stub_data_buffer = NULL;
         SCLogError(SC_ERR_MEM_ALLOC, "Error allocating memory");
         goto end;
     }
+
+    *stub_data_buffer = ptmp;
     memcpy(*stub_data_buffer + *stub_data_buffer_len, input, stub_len);
 
     *stub_data_fresh = 1;
@@ -114,8 +121,9 @@ end:
  * fragmented packets.
  */
 static int DCERPCUDPParseHeader(Flow *f, void *dcerpcudp_state,
-		AppLayerParserState *pstate, uint8_t *input, uint32_t input_len,
-		AppLayerParserResult *output) {
+                                AppLayerParserState *pstate,
+                                uint8_t *input, uint32_t input_len)
+{
 	SCEnter();
 	uint8_t *p = input;
 	DCERPCUDPState *sstate = (DCERPCUDPState *) dcerpcudp_state;
@@ -267,318 +275,397 @@ static int DCERPCUDPParseHeader(Flow *f, void *dcerpcudp_state,
                 /* We fall through to the next case if we still have input.
                  * Same applies for other cases as well */
 			}
+            /* fall through */
 		case 1:
 			sstate->dcerpc.dcerpchdrudp.type = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 2:
 			sstate->dcerpc.dcerpchdrudp.flags1 = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 3:
 			sstate->dcerpc.dcerpchdrudp.flags2 = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 4:
 			sstate->dcerpc.dcerpchdrudp.drep[0] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 5:
 			sstate->dcerpc.dcerpchdrudp.drep[1] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 6:
 			sstate->dcerpc.dcerpchdrudp.drep[2] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 7:
 			sstate->dcerpc.dcerpchdrudp.serial_hi = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 8:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[3] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 9:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[2] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 10:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[1] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 11:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[0] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 12:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[5] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 13:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[4] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 14:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[7] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 15:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[6] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 16:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[8] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 17:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[9] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 18:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[10] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 19:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[11] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 20:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[12] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 21:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[13] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 22:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[14] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 23:
 			sstate->dcerpc.dcerpchdrudp.objectuuid[15] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 24:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[3] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 25:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[2] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 26:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[1] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 27:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[0] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 28:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[5] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 29:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[4] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 30:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[7] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 31:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[6] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 32:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[8] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 33:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[9] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 34:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[10] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 35:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[11] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 36:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[12] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 37:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[13] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 38:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[14] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 39:
 			sstate->dcerpc.dcerpchdrudp.interfaceuuid[15] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 40:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[3] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 41:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[2] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 42:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[1] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 43:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[0] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 44:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[5] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 45:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[4] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 46:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[7] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 47:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[6] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 48:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[8] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 49:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[9] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 50:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[10] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 51:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[11] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 52:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[12] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 53:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[13] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 54:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[14] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 55:
 			sstate->dcerpc.dcerpchdrudp.activityuuid[15] = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 56:
 			sstate->dcerpc.dcerpchdrudp.server_boot = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 57:
 			sstate->dcerpc.dcerpchdrudp.server_boot |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 58:
 			sstate->dcerpc.dcerpchdrudp.server_boot |= *(p++) << 16;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 59:
 			sstate->dcerpc.dcerpchdrudp.server_boot |= *(p++) << 24;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 60:
 			sstate->dcerpc.dcerpchdrudp.if_vers = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 61:
 			sstate->dcerpc.dcerpchdrudp.if_vers |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 62:
 			sstate->dcerpc.dcerpchdrudp.if_vers |= *(p++) << 16;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 63:
 			sstate->dcerpc.dcerpchdrudp.if_vers |= *(p++) << 24;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 64:
 			sstate->dcerpc.dcerpchdrudp.seqnum = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 65:
 			sstate->dcerpc.dcerpchdrudp.seqnum |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 66:
 			sstate->dcerpc.dcerpchdrudp.seqnum |= *(p++) << 16;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 67:
 			sstate->dcerpc.dcerpchdrudp.seqnum |= *(p++) << 24;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 68:
 			sstate->dcerpc.dcerpchdrudp.opnum = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 69:
 			sstate->dcerpc.dcerpchdrudp.opnum |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 70:
 			sstate->dcerpc.dcerpchdrudp.ihint = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 71:
 			sstate->dcerpc.dcerpchdrudp.ihint |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 72:
 			sstate->dcerpc.dcerpchdrudp.ahint = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 73:
 			sstate->dcerpc.dcerpchdrudp.ahint |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 74:
 			sstate->dcerpc.dcerpchdrudp.fraglen = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 75:
 			sstate->dcerpc.dcerpchdrudp.fraglen |= *(p++) << 8;
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 76:
 			sstate->dcerpc.dcerpchdrudp.fragnum = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 77:
 			sstate->dcerpc.dcerpchdrudp.fragnum |= *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 78:
 			sstate->dcerpc.dcerpchdrudp.auth_proto = *(p++);
 			if (!(--input_len))
 				break;
+            /* fall through */
 		case 79:
 			sstate->dcerpc.dcerpchdrudp.serial_lo = *(p++);
 			if (sstate->dcerpc.dcerpchdrudp.drep[0] != 0x10) {
@@ -619,7 +706,7 @@ static int DCERPCUDPParseHeader(Flow *f, void *dcerpcudp_state,
 static int DCERPCUDPParse(Flow *f, void *dcerpc_state,
                           AppLayerParserState *pstate,
                           uint8_t *input, uint32_t input_len,
-                          void *local_data, AppLayerParserResult *output)
+                          void *local_data)
 {
 	uint32_t retval = 0;
 	uint32_t parsed = 0;
@@ -629,7 +716,7 @@ static int DCERPCUDPParse(Flow *f, void *dcerpc_state,
 	DCERPCUDPState *sstate = (DCERPCUDPState *) dcerpc_state;
 	while (sstate->bytesprocessed < DCERPC_UDP_HDR_LEN && input_len) {
 		hdrretval = DCERPCUDPParseHeader(f, dcerpc_state, pstate, input,
-				input_len, output);
+                                         input_len);
 		if (hdrretval == -1 || hdrretval > (int32_t)input_len) {
 			sstate->bytesprocessed = 0;
 			SCReturnInt(hdrretval);
@@ -659,7 +746,7 @@ static int DCERPCUDPParse(Flow *f, void *dcerpc_state,
 			&& sstate->bytesprocessed < sstate->dcerpc.dcerpchdrudp.fraglen
 			&& input_len) {
 		retval = FragmentDataParser(f, dcerpc_state, pstate, input + parsed,
-				input_len, output);
+                                    input_len);
 		if (retval || retval > input_len) {
 			parsed += retval;
 			input_len -= retval;
@@ -677,8 +764,6 @@ static int DCERPCUDPParse(Flow *f, void *dcerpc_state,
 	if (pstate == NULL)
 		SCReturnInt(-1);
 
-	pstate->parse_field = 0;
-
 	SCReturnInt(1);
 }
 
@@ -711,24 +796,50 @@ static void DCERPCUDPStateFree(void *s) {
         sstate->dcerpc.dcerpcresponse.stub_data_buffer = NULL;
         sstate->dcerpc.dcerpcresponse.stub_data_buffer_len = 0;
     }
-	if (s) {
-		SCFree(s);
-		s = NULL;
-	}
+    SCFree(s);
+}
+
+static int DCERPCUDPRegisterPatternsForProtocolDetection(void)
+{
+    if (AppLayerProtoDetectPMRegisterPatternCS(IPPROTO_UDP, ALPROTO_DCERPC,
+                                               "|04 00|", 2, 0, STREAM_TOSERVER) < 0)
+    {
+        return -1;
+    }
+
+    return 0;
 }
 
 void RegisterDCERPCUDPParsers(void) {
-    char *proto_name = "dcerpcudp";
+    char *proto_name = "dcerpc";
 
-    /** DCERPC */
-    AlpProtoAdd(&alp_proto_ctx, proto_name, IPPROTO_UDP, ALPROTO_DCERPC_UDP, "|04 00|", 2, 0, STREAM_TOSERVER);
+    if (AppLayerProtoDetectConfProtoDetectionEnabled("udp", proto_name)) {
+        AppLayerProtoDetectRegisterProtocol(ALPROTO_DCERPC, proto_name);
+        if (DCERPCUDPRegisterPatternsForProtocolDetection() < 0)
+            return;
+    } else {
+        SCLogInfo("Protocol detection and parser disabled for %s protocol.",
+                  "dcerpc");
+        return;
+    }
 
-	AppLayerRegisterProto(proto_name, ALPROTO_DCERPC_UDP, STREAM_TOSERVER,
-			DCERPCUDPParse);
-	AppLayerRegisterProto(proto_name, ALPROTO_DCERPC_UDP, STREAM_TOCLIENT,
-			DCERPCUDPParse);
-	AppLayerRegisterStateFuncs(ALPROTO_DCERPC_UDP, DCERPCUDPStateAlloc,
-			DCERPCUDPStateFree);
+    if (AppLayerParserConfParserEnabled("udp", "dcerpc")) {
+        AppLayerParserRegisterParser(IPPROTO_UDP, ALPROTO_DCERPC, STREAM_TOSERVER,
+                                     DCERPCUDPParse);
+        AppLayerParserRegisterParser(IPPROTO_UDP, ALPROTO_DCERPC, STREAM_TOCLIENT,
+                                     DCERPCUDPParse);
+        AppLayerParserRegisterStateFuncs(IPPROTO_UDP, ALPROTO_DCERPC, DCERPCUDPStateAlloc,
+                                         DCERPCUDPStateFree);
+        AppLayerParserRegisterParserAcceptableDataDirection(IPPROTO_UDP, ALPROTO_DCERPC, STREAM_TOSERVER);
+    } else {
+        SCLogInfo("Parsed disabled for %s protocol. Protocol detection"
+                  "still on.", "dcerpc");
+    }
+#ifdef UNITTESTS
+    AppLayerParserRegisterProtocolUnittests(IPPROTO_UDP, ALPROTO_DCERPC, DCERPCUDPParserRegisterTests);
+#endif
+
+    return;
 }
 
 /* UNITTESTS */
@@ -928,20 +1039,26 @@ int DCERPCUDPParserTest01(void) {
 
 	TcpSession ssn;
 	DCERPCUuidEntry *uuid_entry;
+    AppLayerParserThreadCtx *alp_tctx = AppLayerParserThreadCtxAlloc();
 
 	memset(&f, 0, sizeof(f));
 	memset(&ssn, 0, sizeof(ssn));
-	f.protoctx = (void *)&ssn;
     FLOW_INITIALIZE(&f);
+	f.protoctx = (void *)&ssn;
+    f.proto = IPPROTO_UDP;
+    f.protomap = FlowGetProtoMapping(f.proto);
 
 	StreamTcpInitConfig(TRUE);
 
-	int r = AppLayerParse(NULL, &f, ALPROTO_DCERPC_UDP, STREAM_TOSERVER|STREAM_START, dcerpcrequest, requestlen);
+	SCMutexLock(&f.m);
+	int r = AppLayerParserParse(alp_tctx, &f, ALPROTO_DCERPC, STREAM_TOSERVER|STREAM_START, dcerpcrequest, requestlen);
 	if (r != 0) {
 		printf("dcerpc header check returned %" PRId32 ", expected 0: ", r);
 		result = 0;
+		SCMutexUnlock(&f.m);
 		goto end;
 	}
+	SCMutexUnlock(&f.m);
 
 	DCERPCUDPState *dcerpc_state = f.alstate;
 	if (dcerpc_state == NULL) {
@@ -974,6 +1091,8 @@ int DCERPCUDPParserTest01(void) {
 	}
 
 end:
+    if (alp_tctx != NULL)
+        AppLayerParserThreadCtxFree(alp_tctx);
 	StreamTcpFreeConfig(TRUE);
 	return result;
 }
