@@ -42,9 +42,72 @@
 #include "util-debug.h"
 #include "util-path.h"
 
+/** Maximum size of a complete domain name. */
+#define NODE_NAME_MAX 1024
+
 static ConfNode *root = NULL;
 static ConfNode *root_backup = NULL;
 
+/**
+ * \brief Helper function to get a node, creating it if it does not
+ * exist.
+ *
+ * This function exits on memory failure as creating configuration
+ * nodes is usually part of application initialization.
+ *
+ * \param name The name of the configuration node to get.
+ * \param final Flag to set created nodes as final or not.
+ *
+ * \retval The existing configuration node if it exists, or a newly
+ *   created node for the provided name.  On error, NULL will be returned.
+ */
+static ConfNode *
+ConfGetNodeOrCreate(char *name, int final)
+{
+    ConfNode *parent = root;
+    ConfNode *node = NULL;
+    char node_name[NODE_NAME_MAX];
+    char *key;
+    char *next;
+
+    if (strlcpy(node_name, name, sizeof(node_name)) >= sizeof(node_name)) {
+        SCLogError(SC_ERR_CONF_NAME_TOO_LONG,
+            "Configuration name too long: %s", name);
+        return NULL;
+    }
+
+    key = node_name;
+
+    do {
+        if ((next = strchr(key, '.')) != NULL)
+            *next++ = '\0';
+        if ((node = ConfNodeLookupChild(parent, key)) == NULL) {
+            node = ConfNodeNew();
+            if (unlikely(node == NULL)) {
+                SCLogWarning(SC_ERR_MEM_ALLOC,
+                    "Failed to allocate memory for configuration.");
+                goto end;
+            }
+            node->name = SCStrdup(key);
+            if (unlikely(node->name == NULL)) {
+                ConfNodeFree(node);
+                node = NULL;
+                SCLogWarning(SC_ERR_MEM_ALLOC,
+                    "Failed to allocate memory for configuration.");
+                goto end;
+            }
+            node->parent = parent;
+            node->final = final;
+            TAILQ_INSERT_TAIL(&parent->head, node, next);
+        }
+        key = next;
+        parent = node;
+    } while (next != NULL);
+
+end:
+    return node;
+}
+
 /**
  * \brief Initialize the configuration system.
  */
@@ -79,8 +142,6 @@ ConfNodeNew(void)
     if (unlikely(new == NULL)) {
         return NULL;
     }
-    /* By default we allow an override. */
-    new->allow_override = 1;
     TAILQ_INIT(&new->head);
 
     return new;
@@ -111,45 +172,33 @@ ConfNodeFree(ConfNode *node)
 /**
  * \brief Get a ConfNode by name.
  *
- * \param key The full name of the configuration node to lookup.
+ * \param name The full name of the configuration node to lookup.
  *
  * \retval A pointer to ConfNode is found or NULL if the configuration
  *    node does not exist.
  */
 ConfNode *
-ConfGetNode(char *key)
+ConfGetNode(char *name)
 {
     ConfNode *node = root;
-#if !defined(__WIN32) && !defined(_WIN32)
-    char *saveptr = NULL;
-#endif /* __WIN32 */
-    char *token;
+    char node_name[NODE_NAME_MAX];
+    char *key;
+    char *next;
 
-    /* Need to dup the key for tokenization... */
-    char *tokstr = SCStrdup(key);
-    if (unlikely(tokstr == NULL)) {
+    if (strlcpy(node_name, name, sizeof(node_name)) >= sizeof(node_name)) {
+        SCLogError(SC_ERR_CONF_NAME_TOO_LONG,
+            "Configuration name too long: %s", name);
         return NULL;
     }
 
-#if defined(__WIN32) || defined(_WIN32)
-    token = strtok(tokstr, ".");
-#else
-    token = strtok_r(tokstr, ".", &saveptr);
-#endif /* __WIN32 */
-    for (;;) {
-        node = ConfNodeLookupChild(node, token);
-        if (node == NULL)
-            break;
+    key = node_name;
+    do {
+        if ((next = strchr(key, '.')) != NULL)
+            *next++ = '\0';
+        node = ConfNodeLookupChild(node, key);
+        key = next;
+    } while (next != NULL && node != NULL);
 
-#if defined(__WIN32) || defined(_WIN32)
-        token = strtok(NULL, ".");
-#else
-        token = strtok_r(NULL, ".", &saveptr);
-#endif /* __WIN32 */
-        if (token == NULL)
-            break;
-    }
-    SCFree(tokstr);
     return node;
 }
 
@@ -165,80 +214,59 @@ ConfGetRootNode(void)
 /**
  * \brief Set a configuration value.
  *
+ * Configuration values set with this function may be overridden by
+ * subsequent calls, or if the value appears multiple times in a
+ * configuration file.
+ *
  * \param name The name of the configuration parameter to set.
  * \param val The value of the configuration parameter.
- * \param allow_override Can a subsequent set override this value.
  *
  * \retval 1 if the value was set otherwise 0.
  */
 int
-ConfSet(char *name, char *val, int allow_override)
+ConfSet(char *name, char *val)
 {
-    ConfNode *parent = root;
-    ConfNode *node;
-    char *token;
-#if !defined(__WIN32) && !defined(_WIN32)
-    char *saveptr = NULL;
-#endif /* __WIN32 */
-    /* First check if the node already exists. */
-    node = ConfGetNode(name);
-    if (node != NULL) {
-        if (!node->allow_override) {
-            return 0;
-        }
-        else {
-            if (node->val != NULL)
-                SCFree(node->val);
-            node->val = SCStrdup(val);
-            node->allow_override = allow_override;
-            return 1;
-        }
+    ConfNode *node = ConfGetNodeOrCreate(name, 0);
+    if (node == NULL || node->final) {
+        return 0;
     }
-    else {
-        char *tokstr = SCStrdup(name);
-        if (unlikely(tokstr == NULL)) {
-            return 0;
-        }
-#if defined(__WIN32) || defined(_WIN32)
-        token = strtok(tokstr, ".");
-#else
-        token = strtok_r(tokstr, ".", &saveptr);
-#endif /* __WIN32 */
-        node = ConfNodeLookupChild(parent, token);
-        for (;;) {
-            if (node == NULL) {
-                node = ConfNodeNew();
-                node->name = SCStrdup(token);
-                node->parent = parent;
-                TAILQ_INSERT_TAIL(&parent->head, node, next);
-                parent = node;
-            }
-            else {
-                parent = node;
-            }
-#if defined(__WIN32) || defined(_WIN32)
-            token = strtok(NULL, ".");
-#else
-            token = strtok_r(NULL, ".", &saveptr);
-#endif /* __WIN32 */
-            if (token == NULL) {
-                if (!node->allow_override)
-                    break;
-                if (node->val != NULL)
-                    SCFree(node->val);
-                node->val = SCStrdup(val);
-                node->allow_override = allow_override;
-                break;
-            }
-            else {
-                node = ConfNodeLookupChild(parent, token);
-            }
-        }
-        SCFree(tokstr);
+    if (node->val != NULL)
+        SCFree(node->val);
+    node->val = SCStrdup(val);
+    if (unlikely(node->val == NULL)) {
+        return 0;
     }
+    return 1;
+}
 
-    SCLogDebug("configuration parameter '%s' set", name);
-
+/**
+ * \brief Set a final configuration value.
+ *
+ * A final configuration value is a value that cannot be overridden by
+ * the configuration file.  Its mainly useful for setting values that
+ * are supplied on the command line prior to the configuration file
+ * being loaded.  However, a subsequent call to this function can
+ * override a previously set value.
+ *
+ * \param name The name of the configuration parameter to set.
+ * \param val The value of the configuration parameter.
+ *
+ * \retval 1 if the value was set otherwise 0.
+ */
+int
+ConfSetFinal(char *name, char *val)
+{
+    ConfNode *node = ConfGetNodeOrCreate(name, 1);
+    if (node == NULL) {
+        return 0;
+    }
+    if (node->val != NULL)
+        SCFree(node->val);
+    node->val = SCStrdup(val);
+    if (unlikely(node->val == NULL)) {
+        return 0;
+    }
+    node->final = 1;
     return 1;
 }
 
@@ -623,6 +651,9 @@ ConfNodeDump(ConfNode *node, const char *prefix)
     level++;
     TAILQ_FOREACH(child, &node->head, next) {
         name[level] = SCStrdup(child->name);
+        if (unlikely(name[level] == NULL)) {
+            continue;
+        }
         if (prefix == NULL) {
             printf("%s = %s\n", ConfPrintNameArray(name, level),
                 child->val);
@@ -773,8 +804,41 @@ char *ConfLoadCompleteIncludePath(char *file)
     return path;
 }
 
+/**
+ * \brief Prune a configuration node.
+ *
+ * Pruning a configuration is similar to freeing, but only fields that
+ * may be overridden are, leaving final type parameters.  Additional
+ * the value of the provided node is also free'd, but the node itself
+ * is left.
+ *
+ * \param node The configuration node to prune.
+ */
+void
+ConfNodePrune(ConfNode *node)
+{
+    ConfNode *item, *it;
 
+    for (item = TAILQ_FIRST(&node->head); item != NULL; item = it) {
+        it = TAILQ_NEXT(item, next);
+        if (!item->final) {
+            ConfNodePrune(item);
+            if (TAILQ_EMPTY(&item->head)) {
+                TAILQ_REMOVE(&node->head, item, next);
+                if (item->name != NULL)
+                    SCFree(item->name);
+                if (item->val != NULL)
+                    SCFree(item->val);
+                SCFree(item);
+            }
+        }
+    }
 
+    if (node->val != NULL) {
+        SCFree(node->val);
+        node->val = NULL;
+    }
+}
 
 #ifdef UNITTESTS
 
@@ -800,7 +864,7 @@ ConfTestSetAndGet(void)
     char value[] = "some-value";
     char *value0;
 
-    if (ConfSet(name, value, 1) != 1)
+    if (ConfSet(name, value) != 1)
         return 0;
     if (ConfGet(name, &value0) != 1)
         return 0;
@@ -826,9 +890,9 @@ ConfTestOverrideValue1(void)
     char *val;
     int rc;
 
-    if (ConfSet(name, value0, 1) != 1)
+    if (ConfSet(name, value0) != 1)
         return 0;
-    if (ConfSet(name, value1, 1) != 1)
+    if (ConfSet(name, value1) != 1)
         return 0;
     if (ConfGet(name, &val) != 1)
         return 0;
@@ -842,8 +906,7 @@ ConfTestOverrideValue1(void)
 }
 
 /**
- * Test that overriding a value is not allowed provided that
- * allow_override is false and make sure the value was not overrided.
+ * Test that a final value will not be overrided by a ConfSet.
  */
 static int
 ConfTestOverrideValue2(void)
@@ -854,9 +917,9 @@ ConfTestOverrideValue2(void)
     char *val;
     int rc;
 
-    if (ConfSet(name, value0, 0) != 1)
+    if (ConfSetFinal(name, value0) != 1)
         return 0;
-    if (ConfSet(name, value1, 1) != 0)
+    if (ConfSet(name, value1) != 0)
         return 0;
     if (ConfGet(name, &val) != 1)
         return 0;
@@ -878,7 +941,7 @@ ConfTestGetInt(void)
     char name[] = "some-int.x";
     intmax_t val;
 
-    if (ConfSet(name, "0", 1) != 1)
+    if (ConfSet(name, "0") != 1)
         return 0;
     if (ConfGetInt(name, &val) != 1)
         return 0;
@@ -886,21 +949,21 @@ ConfTestGetInt(void)
     if (val != 0)
         return 0;
 
-    if (ConfSet(name, "-1", 1) != 1)
+    if (ConfSet(name, "-1") != 1)
         return 0;
     if (ConfGetInt(name, &val) != 1)
         return 0;
     if (val != -1)
         return 0;
 
-    if (ConfSet(name, "0xffff", 1) != 1)
+    if (ConfSet(name, "0xffff") != 1)
         return 0;
     if (ConfGetInt(name, &val) != 1)
         return 0;
     if (val != 0xffff)
         return 0;
 
-    if (ConfSet(name, "not-an-int", 1) != 1)
+    if (ConfSet(name, "not-an-int") != 1)
         return 0;
     if (ConfGetInt(name, &val) != 0)
         return 0;
@@ -932,7 +995,7 @@ ConfTestGetBool(void)
     size_t u;
 
     for (u = 0; u < sizeof(trues) / sizeof(trues[0]); u++) {
-        if (ConfSet(name, trues[u], 1) != 1)
+        if (ConfSet(name, trues[u]) != 1)
             return 0;
         if (ConfGetBool(name, &val) != 1)
             return 0;
@@ -941,7 +1004,7 @@ ConfTestGetBool(void)
     }
 
     for (u = 0; u < sizeof(falses) / sizeof(falses[0]); u++) {
-        if (ConfSet(name, falses[u], 1) != 1)
+        if (ConfSet(name, falses[u]) != 1)
             return 0;
         if (ConfGetBool(name, &val) != 1)
             return 0;
@@ -1047,13 +1110,13 @@ ConfNodeLookupChildValueTest(void)
 
 static int ConfGetChildValueWithDefaultTest(void)
 {
-    char  *val;
+    char  *val = "";
     int ret = 1;
     ConfCreateContextBackup();
     ConfInit();
-    ConfSet("af-packet.0.interface", "eth0", 1);
-    ConfSet("af-packet.1.interface", "default", 1);
-    ConfSet("af-packet.1.cluster-type", "cluster_cpu", 1);
+    ConfSet("af-packet.0.interface", "eth0");
+    ConfSet("af-packet.1.interface", "default");
+    ConfSet("af-packet.1.cluster-type", "cluster_cpu");
 
     ConfNode *root = ConfGetNode("af-packet.0");
     ConfNode *dflt = ConfGetNode("af-packet.1");
@@ -1064,7 +1127,7 @@ static int ConfGetChildValueWithDefaultTest(void)
         return 0;
     }
 
-    ConfSet("af-packet.0.cluster-type", "cluster_flow", 1);
+    ConfSet("af-packet.0.cluster-type", "cluster_flow");
     ConfGetChildValueWithDefault(root, dflt, "cluster-type", &val);
 
     if (strcmp(val, "cluster_flow")) {
@@ -1077,12 +1140,12 @@ static int ConfGetChildValueWithDefaultTest(void)
 
 static int ConfGetChildValueIntWithDefaultTest(void)
 {
-    intmax_t val;
+    intmax_t val = 0;
     ConfCreateContextBackup();
     ConfInit();
-    ConfSet("af-packet.0.interface", "eth0", 1);
-    ConfSet("af-packet.1.interface", "default", 1);
-    ConfSet("af-packet.1.threads", "2", 1);
+    ConfSet("af-packet.0.interface", "eth0");
+    ConfSet("af-packet.1.interface", "default");
+    ConfSet("af-packet.1.threads", "2");
 
     ConfNode *root = ConfGetNode("af-packet.0");
     ConfNode *dflt = ConfGetNode("af-packet.1");
@@ -1093,7 +1156,7 @@ static int ConfGetChildValueIntWithDefaultTest(void)
         return 0;
     }
 
-    ConfSet("af-packet.0.threads", "1", 1);
+    ConfSet("af-packet.0.threads", "1");
     ConfGetChildValueIntWithDefault(root, dflt, "threads", &val);
 
     ConfDeInit();
@@ -1109,9 +1172,9 @@ static int ConfGetChildValueBoolWithDefaultTest(void)
     int val;
     ConfCreateContextBackup();
     ConfInit();
-    ConfSet("af-packet.0.interface", "eth0", 1);
-    ConfSet("af-packet.1.interface", "default", 1);
-    ConfSet("af-packet.1.use-mmap", "yes", 1);
+    ConfSet("af-packet.0.interface", "eth0");
+    ConfSet("af-packet.1.interface", "default");
+    ConfSet("af-packet.1.use-mmap", "yes");
 
     ConfNode *root = ConfGetNode("af-packet.0");
     ConfNode *dflt = ConfGetNode("af-packet.1");
@@ -1122,7 +1185,7 @@ static int ConfGetChildValueBoolWithDefaultTest(void)
         return 0;
     }
 
-    ConfSet("af-packet.0.use-mmap", "no", 1);
+    ConfSet("af-packet.0.use-mmap", "no");
     ConfGetChildValueBoolWithDefault(root, dflt, "use-mmap", &val);
 
     ConfDeInit();
@@ -1142,7 +1205,7 @@ ConfNodeRemoveTest(void)
     ConfCreateContextBackup();
     ConfInit();
 
-    if (ConfSet("some.nested.parameter", "blah", 1) != 1)
+    if (ConfSet("some.nested.parameter", "blah") != 1)
         return 0;
 
     ConfNode *node = ConfGetNode("some.nested.parameter");
@@ -1167,7 +1230,7 @@ ConfSetTest(void)
     ConfInit();
 
     /* Set some value with 2 levels. */
-    if (ConfSet("one.two", "three", 1) != 1)
+    if (ConfSet("one.two", "three") != 1)
         return 0;
     ConfNode *n = ConfGetNode("one.two");
     if (n == NULL)
@@ -1176,7 +1239,7 @@ ConfSetTest(void)
     /* Set another 2 level parameter with the same first level, this
      * used to trigger a bug that caused the second level of the name
      * to become a first level node. */
-    if (ConfSet("one.three", "four", 1) != 1)
+    if (ConfSet("one.three", "four") != 1)
         return 0;
 
     n = ConfGetNode("one.three");
@@ -1194,6 +1257,146 @@ ConfSetTest(void)
     return 1;
 }
 
+static int
+ConfGetNodeOrCreateTest(void)
+{
+    ConfNode *node;
+    int ret = 0;
+
+    ConfCreateContextBackup();
+    ConfInit();
+
+    /* Get a node that should not exist, give it a value, re-get it
+     * and make sure the second time it returns the existing node. */
+    node = ConfGetNodeOrCreate("node0", 0);
+    if (node == NULL) {
+        fprintf(stderr, "returned null\n");
+        goto end;
+    }
+    if (node->parent == NULL || node->parent != root) {
+        fprintf(stderr, "unexpected parent node\n");
+        goto end;
+    }
+    if (node->val != NULL) {
+        fprintf(stderr, "node already existed\n");
+        goto end;
+    }
+    node->val = SCStrdup("node0");
+    node = ConfGetNodeOrCreate("node0", 0);
+    if (node == NULL) {
+        fprintf(stderr, "returned null\n");
+        goto end;
+    }
+    if (node->val == NULL) {
+        fprintf(stderr, "new node was allocated\n");
+        goto end;
+    }
+    if (strcmp(node->val, "node0") != 0) {
+        fprintf(stderr, "node did not have expected value\n");
+        goto end;
+    }
+
+    /* Do the same, but for something deeply nested. */
+    node = ConfGetNodeOrCreate("parent.child.grandchild", 0);
+    if (node == NULL) {
+        fprintf(stderr, "returned null\n");
+        goto end;
+    }
+    if (node->parent == NULL || node->parent == root) {
+        fprintf(stderr, "unexpected parent node\n");
+        goto end;
+    }
+    if (node->val != NULL) {
+        fprintf(stderr, "node already existed\n");
+        goto end;
+    }
+    node->val = SCStrdup("parent.child.grandchild");
+    node = ConfGetNodeOrCreate("parent.child.grandchild", 0);
+    if (node == NULL) {
+        fprintf(stderr, "returned null\n");
+        goto end;
+    }
+    if (node->val == NULL) {
+        fprintf(stderr, "new node was allocated\n");
+        goto end;
+    }
+    if (strcmp(node->val, "parent.child.grandchild") != 0) {
+        fprintf(stderr, "node did not have expected value\n");
+        goto end;
+    }
+
+    /* Test that 2 child nodes have the same root. */
+    ConfNode *child1 = ConfGetNodeOrCreate("parent.kids.child1", 0);
+    ConfNode *child2 = ConfGetNodeOrCreate("parent.kids.child2", 0);
+    if (child1 == NULL || child2 == NULL) {
+        fprintf(stderr, "returned null\n");
+        goto end;
+    }
+    if (child1->parent != child2->parent) {
+        fprintf(stderr, "child nodes have different parents\n");
+        goto end;
+    }
+    if (strcmp(child1->parent->name, "kids") != 0) {
+        fprintf(stderr, "parent node had unexpected name\n");
+        goto end;
+    }
+
+    ret = 1;
+
+end:
+    ConfDeInit();
+    ConfRestoreContextBackup();
+
+    return ret;
+}
+
+static int
+ConfNodePruneTest(void)
+{
+    int ret = 0;
+    ConfNode *node;
+
+    ConfCreateContextBackup();
+    ConfInit();
+
+    /* Test that final nodes exist after a prune. */
+    if (ConfSet("node.notfinal", "notfinal") != 1)
+        goto end;
+    if (ConfSetFinal("node.final", "final") != 1)
+        goto end;
+    if (ConfGetNode("node.notfinal") == NULL)
+        goto end;
+    if (ConfGetNode("node.final") == NULL)
+        goto end;
+    if ((node = ConfGetNode("node")) == NULL)
+        goto end;
+    ConfNodePrune(node);
+    if (ConfGetNode("node.notfinal") != NULL)
+        goto end;
+    if (ConfGetNode("node.final") == NULL)
+        goto end;
+
+    /* Test that everything under a final node exists after a prune. */
+    if (ConfSet("node.final.one", "one") != 1)
+        goto end;
+    if (ConfSet("node.final.two", "two") != 1)
+        goto end;
+    ConfNodePrune(node);
+    if (ConfNodeLookupChild(node, "final") == NULL)
+        goto end;
+    if (ConfGetNode("node.final.one") == NULL)
+        goto end;
+    if (ConfGetNode("node.final.two") == NULL)
+        goto end;
+
+    ret = 1;
+
+end:
+    ConfDeInit();
+    ConfRestoreContextBackup();
+
+    return ret;
+}
 
 void
 ConfRegisterTests(void)
@@ -1211,6 +1414,8 @@ ConfRegisterTests(void)
     UtRegisterTest("ConfGetChildValueWithDefaultTest", ConfGetChildValueWithDefaultTest, 1);
     UtRegisterTest("ConfGetChildValueIntWithDefaultTest", ConfGetChildValueIntWithDefaultTest, 1);
     UtRegisterTest("ConfGetChildValueBoolWithDefaultTest", ConfGetChildValueBoolWithDefaultTest, 1);
+    UtRegisterTest("ConfGetNodeOrCreateTest", ConfGetNodeOrCreateTest, 1);
+    UtRegisterTest("ConfNodePruneTest", ConfNodePruneTest, 1);
 }
 
 #endif /* UNITTESTS */
