@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2013 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -38,14 +38,19 @@
 #include "util-unittest.h"
 #include "util-debug.h"
 
-void DecodeRaw(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, uint16_t len, PacketQueue *pq)
+#include "pkt-var.h"
+#include "util-profiling.h"
+#include "host.h"
+
+
+int DecodeRaw(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, uint16_t len, PacketQueue *pq)
 {
     SCPerfCounterIncr(dtv->counter_raw, tv->sc_perf_pca);
 
     /* If it is ipv4 or ipv6 it should at least be the size of ipv4 */
-    if (len < IPV4_HEADER_LEN) {
-        ENGINE_SET_EVENT(p,IPV4_PKT_TOO_SMALL);
-        return;
+    if (unlikely(len < IPV4_HEADER_LEN)) {
+        ENGINE_SET_INVALID_EVENT(p, IPV4_PKT_TOO_SMALL);
+        return TM_ECODE_FAILED;
     }
 
     if (IP_GET_RAW_VER(pkt) == 4) {
@@ -58,7 +63,7 @@ void DecodeRaw(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, u
         SCLogDebug("Unknown ip version %" PRIu8 "", IP_GET_RAW_VER(pkt));
         ENGINE_SET_EVENT(p,IPRAW_INVALID_IPV);
     }
-    return;
+    return TM_ECODE_OK;
 }
 
 #ifdef UNITTESTS
@@ -83,16 +88,14 @@ static int DecodeRawTest01 (void)   {
         0x29, 0x9c, 0x00, 0x00, 0x02, 0x04, 0x05, 0x8c,
         0x04, 0x02, 0x08, 0x0a, 0x00, 0xdd, 0x1a, 0x39,
         0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0x02 };
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    Packet *p = PacketGetFromAlloc();
     if (unlikely(p == NULL))
-    return 0;
+        return 0;
     ThreadVars tv;
     DecodeThreadVars dtv;
 
     memset(&dtv, 0, sizeof(DecodeThreadVars));
     memset(&tv,  0, sizeof(ThreadVars));
-    memset(p, 0, SIZE_OF_PACKET);
-    p->pkt = (uint8_t *)(p + 1);
 
     if (PacketCopyData(p, raw_ip, sizeof(raw_ip)) == -1) {
     SCFree(p);
@@ -109,6 +112,7 @@ static int DecodeRawTest01 (void)   {
         return 1;
     }
 
+    PACKET_RECYCLE(p);
     FlowShutdown();
     SCFree(p);
     return 0;
@@ -129,16 +133,14 @@ static int DecodeRawTest02 (void)   {
         0x70, 0x02, 0x40, 0x00, 0xb8, 0xc8, 0x00, 0x00,
         0x02, 0x04, 0x05, 0xb4, 0x01, 0x01, 0x04, 0x02 };
 
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    Packet *p = PacketGetFromAlloc();
     if (unlikely(p == NULL))
-    return 0;
+        return 0;
     ThreadVars tv;
     DecodeThreadVars dtv;
 
     memset(&dtv, 0, sizeof(DecodeThreadVars));
     memset(&tv,  0, sizeof(ThreadVars));
-    memset(p, 0, SIZE_OF_PACKET);
-    p->pkt = (uint8_t *)(p + 1);
 
     if (PacketCopyData(p, raw_ip, sizeof(raw_ip)) == -1) {
     SCFree(p);
@@ -148,13 +150,16 @@ static int DecodeRawTest02 (void)   {
     FlowInitConfig(FLOW_QUIET);
 
     DecodeRaw(&tv, &dtv, p, raw_ip, GET_PKT_LEN(p), NULL);
-    FlowShutdown();
     if (p->ip4h == NULL) {
         printf("expected a valid ipv4 header but it was NULL: ");
+        PACKET_RECYCLE(p);
+        FlowShutdown();
         SCFree(p);
         return 1;
     }
 
+    PACKET_RECYCLE(p);
+    FlowShutdown();
     SCFree(p);
     return 0;
 }
@@ -175,20 +180,18 @@ static int DecodeRawTest03 (void)   {
         0x34, 0x40, 0x67, 0x31, 0x3b, 0x63, 0x61, 0x74,
         0x20, 0x6b, 0x65, 0x79, 0x3b };
 
-    Packet *p = SCMalloc(SIZE_OF_PACKET);
+    Packet *p = PacketGetFromAlloc();
     if (unlikely(p == NULL))
-    return 0;
+        return 0;
     ThreadVars tv;
     DecodeThreadVars dtv;
 
     memset(&dtv, 0, sizeof(DecodeThreadVars));
     memset(&tv,  0, sizeof(ThreadVars));
-    memset(p, 0, SIZE_OF_PACKET);
-    p->pkt = (uint8_t *)(p + 1);
 
     if (PacketCopyData(p, raw_ip, sizeof(raw_ip)) == -1) {
-    SCFree(p);
-    return 1;
+        SCFree(p);
+        return 1;
     }
 
     FlowInitConfig(FLOW_QUIET);
@@ -201,6 +204,7 @@ static int DecodeRawTest03 (void)   {
     } else {
         printf("expected IPRAW_INVALID_IPV to be set but it wasn't: ");
     }
+    PACKET_RECYCLE(p);
     FlowShutdown();
     SCFree(p);
     return 1;
