@@ -58,11 +58,11 @@
 #define FLOW_IPPROTO_ICMP_EMERG_EST_TIMEOUT 100
 
 enum {
-    FLOW_PROTO_DEFAULT = 0,
-    FLOW_PROTO_TCP,
+    FLOW_PROTO_TCP = 0,
     FLOW_PROTO_UDP,
     FLOW_PROTO_ICMP,
     FLOW_PROTO_SCTP,
+    FLOW_PROTO_DEFAULT,
 
     /* should be last */
     FLOW_PROTO_MAX,
