@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2013 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -30,13 +30,32 @@ enum {
     RUNMODE_PCAP_FILE,
     RUNMODE_PFRING,
     RUNMODE_NFQ,
+    RUNMODE_NFLOG,
     RUNMODE_IPFW,
     RUNMODE_ERF_FILE,
     RUNMODE_DAG,
     RUNMODE_AFP_DEV,
+    RUNMODE_TILERA_MPIPE,
     RUNMODE_UNITTEST,
     RUNMODE_NAPATECH,
     RUNMODE_UNIX_SOCKET,
+    RUNMODE_USER_MAX, /* Last standard running mode */
+    RUNMODE_LIST_KEYWORDS,
+    RUNMODE_LIST_APP_LAYERS,
+    RUNMODE_LIST_CUDA_CARDS,
+    RUNMODE_LIST_RUNMODES,
+    RUNMODE_PRINT_VERSION,
+    RUNMODE_PRINT_BUILDINFO,
+    RUNMODE_PRINT_USAGE,
+    RUNMODE_DUMP_CONFIG,
+    RUNMODE_CONF_TEST,
+    RUNMODE_LIST_UNITTEST,
+    RUNMODE_ENGINE_ANALYSIS,
+#ifdef OS_WIN32
+    RUNMODE_INSTALL_SERVICE,
+    RUNMODE_REMOVE_SERVICE,
+    RUNMODE_CHANGE_SERVICE_PARAMS,
+#endif
     RUNMODE_MAX,
 };
 
@@ -56,12 +75,14 @@ void RunModeShutDown(void);
 #include "runmode-pcap.h"
 #include "runmode-pcap-file.h"
 #include "runmode-pfring.h"
+#include "runmode-tile.h"
 #include "runmode-nfq.h"
 #include "runmode-ipfw.h"
 #include "runmode-erf-file.h"
 #include "runmode-erf-dag.h"
 #include "runmode-napatech.h"
 #include "runmode-af-packet.h"
+#include "runmode-nflog.h"
 #include "runmode-unix-socket.h"
 
 int threading_set_cpu_affinity;
