@@ -31,13 +31,16 @@
 #include "tm-threads.h"
 
 #include "util-privs.h"
+#include "util-device.h"
 #include "tmqh-packetpool.h"
 
 #ifndef HAVE_DAG
 
 TmEcode NoErfDagSupportExit(ThreadVars *, void *, void **);
 
-void TmModuleReceiveErfDagRegister (void) {
+void
+TmModuleReceiveErfDagRegister(void)
+{
     tmm_modules[TMM_RECEIVEERFDAG].name = "ReceiveErfDag";
     tmm_modules[TMM_RECEIVEERFDAG].ThreadInit = NoErfDagSupportExit;
     tmm_modules[TMM_RECEIVEERFDAG].Func = NULL;
@@ -48,7 +51,9 @@ void TmModuleReceiveErfDagRegister (void) {
     tmm_modules[TMM_RECEIVEERFDAG].flags = TM_FLAG_RECEIVE_TM;
 }
 
-void TmModuleDecodeErfDagRegister (void) {
+void
+TmModuleDecodeErfDagRegister(void)
+{
     tmm_modules[TMM_DECODEERFDAG].name = "DecodeErfDag";
     tmm_modules[TMM_DECODEERFDAG].ThreadInit = NoErfDagSupportExit;
     tmm_modules[TMM_DECODEERFDAG].Func = NULL;
@@ -59,11 +64,12 @@ void TmModuleDecodeErfDagRegister (void) {
     tmm_modules[TMM_DECODEERFDAG].flags = TM_FLAG_DECODE_TM;
 }
 
-TmEcode NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
+TmEcode
+NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
 {
     SCLogError(SC_ERR_DAG_NOSUPPORT,
-               "Error creating thread %s: you do not have support for DAG cards "
-               "enabled please recompile with --enable-dag", tv->name);
+        "Error creating thread %s: you do not have support for DAG cards "
+        "enabled please recompile with --enable-dag", tv->name);
     exit(EXIT_FAILURE);
 }
 
@@ -72,6 +78,18 @@ TmEcode NoErfDagSupportExit(ThreadVars *tv, void *initdata, void **data)
 #include "source-erf-dag.h"
 #include <dagapi.h>
 
+/* Minimum amount of data to read from the DAG at a time. */
+#define MINDATA 32768
+
+/* Maximum time (us) to wait for MINDATA to be read. */
+#define MAXWAIT 20000
+
+/* Poll interval in microseconds. */
+#define POLL_INTERVAL 1000;
+
+/* Number of bytes per loop to process before fetching more data. */
+#define BYTES_PER_LOOP (4 * 1024 * 1024) /* 4 MB */
+
 extern int max_pending_packets;
 extern uint8_t suricata_ctl_flags;
 
@@ -85,6 +103,8 @@ typedef struct ErfDagThreadVars_ {
 
     struct timeval maxwait, poll;   /* Could possibly be made static */
 
+    LiveDevice *livedev;
+
     uint64_t bytes;
     uint16_t packets;
     uint16_t drops;
@@ -103,9 +123,10 @@ TmEcode ReceiveErfDagLoop(ThreadVars *, void *data, void *slot);
 TmEcode ReceiveErfDagThreadInit(ThreadVars *, void *, void **);
 void ReceiveErfDagThreadExitStats(ThreadVars *, void *);
 TmEcode ReceiveErfDagThreadDeinit(ThreadVars *, void *);
-
 TmEcode DecodeErfDagThreadInit(ThreadVars *, void *, void **);
-TmEcode DecodeErfDag(ThreadVars *, Packet *, void *, PacketQueue *, PacketQueue *);
+TmEcode DecodeErfDagThreadDeinit(ThreadVars *tv, void *data);
+TmEcode DecodeErfDag(ThreadVars *, Packet *, void *, PacketQueue *,
+    PacketQueue *);
 void ReceiveErfDagCloseStream(int dagfd, int stream);
 
 /**
@@ -136,7 +157,7 @@ TmModuleDecodeErfDagRegister(void)
     tmm_modules[TMM_DECODEERFDAG].ThreadInit = DecodeErfDagThreadInit;
     tmm_modules[TMM_DECODEERFDAG].Func = DecodeErfDag;
     tmm_modules[TMM_DECODEERFDAG].ThreadExitPrintStats = NULL;
-    tmm_modules[TMM_DECODEERFDAG].ThreadDeinit = NULL;
+    tmm_modules[TMM_DECODEERFDAG].ThreadDeinit = DecodeErfDagThreadDeinit;
     tmm_modules[TMM_DECODEERFDAG].RegisterTests = NULL;
     tmm_modules[TMM_DECODEERFDAG].cap_flags = 0;
     tmm_modules[TMM_DECODEERFDAG].flags = TM_FLAG_DECODE_TM;
@@ -165,13 +186,15 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
     int stream_count = 0;
 
     if (initdata == NULL) {
-        SCLogError(SC_ERR_INVALID_ARGUMENT, "Error: No DAG interface provided.");
+        SCLogError(SC_ERR_INVALID_ARGUMENT,
+            "Error: No DAG interface provided.");
         SCReturnInt(TM_ECODE_FAILED);
     }
 
     ErfDagThreadVars *ewtn = SCMalloc(sizeof(ErfDagThreadVars));
     if (unlikely(ewtn == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate memory for ERF DAG thread vars.");
+        SCLogError(SC_ERR_MEM_ALLOC,
+            "Failed to allocate memory for ERF DAG thread vars.");
         exit(EXIT_FAILURE);
     }
 
@@ -181,22 +204,28 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
      * to open for this thread.
      */
     if (dag_parse_name(initdata, ewtn->dagname, DAGNAME_BUFSIZE,
-                       &ewtn->dagstream) < 0)
-    {
+            &ewtn->dagstream) < 0) {
         SCLogError(SC_ERR_INVALID_ARGUMENT,
-                   "Failed to parse DAG interface: %s",
-                   (char*)initdata);
+            "Failed to parse DAG interface: %s",
+            (char*)initdata);
         SCFree(ewtn);
         exit(EXIT_FAILURE);
     }
 
+    ewtn->livedev = LiveGetDevice(initdata);
+    if (ewtn->livedev == NULL) {
+        SCLogError(SC_ERR_INVALID_VALUE, "Unable to get %s live device",
+            (char *)initdata);
+        SCFree(ewtn);
+        SCReturnInt(TM_ECODE_FAILED);
+    }
+
     SCLogInfo("Opening DAG: %s on stream: %d for processing",
         ewtn->dagname, ewtn->dagstream);
 
-    if ((ewtn->dagfd = dag_open(ewtn->dagname)) < 0)
-    {
+    if ((ewtn->dagfd = dag_open(ewtn->dagname)) < 0) {
         SCLogError(SC_ERR_ERF_DAG_OPEN_FAILED, "Failed to open DAG: %s",
-                   ewtn->dagname);
+            ewtn->dagname);
         SCFree(ewtn);
         SCReturnInt(TM_ECODE_FAILED);
     }
@@ -204,11 +233,10 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
     /* Check to make sure the card has enough available streams to
      * support reading from the one specified.
      */
-    if ((stream_count = dag_rx_get_stream_count(ewtn->dagfd)) < 0)
-    {
+    if ((stream_count = dag_rx_get_stream_count(ewtn->dagfd)) < 0) {
         SCLogError(SC_ERR_ERF_DAG_OPEN_FAILED,
-                   "Failed to open stream: %d, DAG: %s, could not query stream count",
-                   ewtn->dagstream, ewtn->dagname);
+            "Failed to open stream: %d, DAG: %s, could not query stream count",
+            ewtn->dagstream, ewtn->dagname);
         SCFree(ewtn);
         SCReturnInt(TM_ECODE_FAILED);
     }
@@ -216,11 +244,10 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
     /* Check to make sure we have enough rx streams to open the stream
      * the user is asking for.
      */
-    if (ewtn->dagstream > stream_count*2)
-    {
+    if (ewtn->dagstream > stream_count * 2) {
         SCLogError(SC_ERR_ERF_DAG_OPEN_FAILED,
-                   "Failed to open stream: %d, DAG: %s, insufficient streams: %d",
-                   ewtn->dagstream, ewtn->dagname, stream_count);
+            "Failed to open stream: %d, DAG: %s, insufficient streams: %d",
+            ewtn->dagstream, ewtn->dagname, stream_count);
         SCFree(ewtn);
         SCReturnInt(TM_ECODE_FAILED);
     }
@@ -228,32 +255,29 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
     /* If we are transmitting into a soft DAG card then set the stream
      * to act in reverse mode.
      */
-    if (0 != (ewtn->dagstream & 0x01))
-    {
+    if (0 != (ewtn->dagstream & 0x01)) {
         /* Setting reverse mode for using with soft dag from daemon side */
-        if(dag_set_mode(ewtn->dagfd, ewtn->dagstream, DAG_REVERSE_MODE)) {
+        if (dag_set_mode(ewtn->dagfd, ewtn->dagstream, DAG_REVERSE_MODE)) {
             SCLogError(SC_ERR_ERF_DAG_STREAM_OPEN_FAILED,
-                       "Failed to set mode to DAG_REVERSE_MODE on stream: %d, DAG: %s",
-                       ewtn->dagstream, ewtn->dagname);
+                "Failed to set mode to DAG_REVERSE_MODE on stream: %d, DAG: %s",
+                ewtn->dagstream, ewtn->dagname);
             SCFree(ewtn);
             SCReturnInt(TM_ECODE_FAILED);
         }
     }
 
-    if (dag_attach_stream(ewtn->dagfd, ewtn->dagstream, 0, 0) < 0)
-    {
+    if (dag_attach_stream(ewtn->dagfd, ewtn->dagstream, 0, 0) < 0) {
         SCLogError(SC_ERR_ERF_DAG_STREAM_OPEN_FAILED,
-                   "Failed to open DAG stream: %d, DAG: %s",
-                   ewtn->dagstream, ewtn->dagname);
+            "Failed to open DAG stream: %d, DAG: %s",
+            ewtn->dagstream, ewtn->dagname);
         SCFree(ewtn);
         SCReturnInt(TM_ECODE_FAILED);
     }
 
-    if (dag_start_stream(ewtn->dagfd, ewtn->dagstream) < 0)
-    {
+    if (dag_start_stream(ewtn->dagfd, ewtn->dagstream) < 0) {
         SCLogError(SC_ERR_ERF_DAG_STREAM_START_FAILED,
-                   "Failed to start DAG stream: %d, DAG: %s",
-                   ewtn->dagstream, ewtn->dagname);
+            "Failed to start DAG stream: %d, DAG: %s",
+            ewtn->dagstream, ewtn->dagname);
         SCFree(ewtn);
         SCReturnInt(TM_ECODE_FAILED);
     }
@@ -265,18 +289,18 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
      * Initialise DAG Polling parameters.
      */
     timerclear(&ewtn->maxwait);
-    ewtn->maxwait.tv_usec = 20 * 1000; /* 20ms timeout */
+    ewtn->maxwait.tv_usec = MAXWAIT;
     timerclear(&ewtn->poll);
-    ewtn->poll.tv_usec = 1 * 1000; /* 1ms poll interval */
+    ewtn->poll.tv_usec = POLL_INTERVAL;
 
     /* 32kB minimum data to return -- we still restrict the number of
      * pkts that are processed to a maximum of dag_max_read_packets.
      */
-    if (dag_set_stream_poll(ewtn->dagfd, ewtn->dagstream, 32*1024, &(ewtn->maxwait), &(ewtn->poll)) < 0)
-    {
+    if (dag_set_stream_poll(ewtn->dagfd, ewtn->dagstream, MINDATA,
+            &(ewtn->maxwait), &(ewtn->poll)) < 0) {
         SCLogError(SC_ERR_ERF_DAG_STREAM_SET_FAILED,
-                   "Failed to set poll parameters for stream: %d, DAG: %s",
-                   ewtn->dagstream, ewtn->dagname);
+            "Failed to set poll parameters for stream: %d, DAG: %s",
+            ewtn->dagstream, ewtn->dagname);
         SCFree(ewtn);
         SCReturnInt(TM_ECODE_FAILED);
     }
@@ -290,7 +314,7 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
     *data = (void *)ewtn;
 
     SCLogInfo("Starting processing packets from stream: %d on DAG: %s",
-              ewtn->dagstream, ewtn->dagname);
+        ewtn->dagstream, ewtn->dagname);
 
     SCReturnInt(TM_ECODE_OK);
 }
@@ -305,7 +329,8 @@ ReceiveErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
  * \retval TM_ECODE_OK on success
  * \retval TM_ECODE_FAILED on failure
  */
-TmEcode ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
+TmEcode
+ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
 {
     SCEnter();
 
@@ -318,8 +343,7 @@ TmEcode ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
 
     dtv->slot = s->slot_next;
 
-    while (1)
-    {
+    while (1) {
         if (suricata_ctl_flags & (SURICATA_STOP | SURICATA_KILL)) {
             SCReturnInt(TM_ECODE_OK);
         }
@@ -334,9 +358,9 @@ TmEcode ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
                 continue;
             } else {
                 SCLogError(SC_ERR_ERF_DAG_STREAM_READ_FAILED,
-                           "Failed to read from stream: %d, DAG: %s when "
-                           "using dag_advance_stream",
-                           dtv->dagstream, dtv->dagname);
+                    "Failed to read from stream: %d, DAG: %s when "
+                    "using dag_advance_stream",
+                    dtv->dagstream, dtv->dagname);
                 SCReturnInt(TM_ECODE_FAILED);
             }
         }
@@ -352,16 +376,16 @@ TmEcode ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
 
         if (err == TM_ECODE_FAILED) {
             SCLogError(SC_ERR_ERF_DAG_STREAM_READ_FAILED,
-                       "Failed to read from stream: %d, DAG: %s",
-                       dtv->dagstream, dtv->dagname);
+                "Failed to read from stream: %d, DAG: %s",
+                dtv->dagstream, dtv->dagname);
             ReceiveErfDagCloseStream(dtv->dagfd, dtv->dagstream);
             SCReturnInt(TM_ECODE_FAILED);
         }
 
-        SCPerfSyncCountersIfSignalled(tv, 0);
+        SCPerfSyncCountersIfSignalled(tv);
 
         SCLogDebug("Read %d records from stream: %d, DAG: %s",
-                   pkts_read, dtv->dagstream, dtv->dagname);
+            pkts_read, dtv->dagstream, dtv->dagname);
     }
 
     SCReturnInt(TM_ECODE_OK);
@@ -373,8 +397,8 @@ TmEcode ReceiveErfDagLoop(ThreadVars *tv, void *data, void *slot)
  * This function takes a pointer to buffer read from the DAG interface
  * and processes it individual records.
  */
-static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
-    uint32_t *pkts_read)
+static inline TmEcode
+ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top, uint32_t *pkts_read)
 {
     SCEnter();
 
@@ -389,7 +413,7 @@ static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
     *pkts_read = 0;
 
     while (((top - ewtn->btm) >= dag_record_size) &&
-        ((processed + dag_record_size) < 4*1024*1024)) {
+        ((processed + dag_record_size) < BYTES_PER_LOOP)) {
 
         /* Make sure we have at least one packet in the packet pool,
          * to prevent us from alloc'ing packets at line rate. */
@@ -405,8 +429,8 @@ static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
         rlen = ntohs(dr->rlen);
         hdr_type = dr->type;
 
-        /* If we don't have enough data to finsih processing this ERF record
-         * return and maybe next time we will.
+        /* If we don't have enough data to finish processing this ERF
+         * record return and maybe next time we will.
          */
         if ((top - ewtn->btm) < rlen)
             SCReturnInt(TM_ECODE_OK);
@@ -427,7 +451,8 @@ static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
             break;
         case TYPE_ETH:
             if (dr->lctr) {
-                SCPerfCounterIncr(ewtn->drops, ewtn->tv->sc_perf_pca);
+                SCPerfCounterAddUI64(ewtn->drops, ewtn->tv->sc_perf_pca,
+                    ntohs(dr->lctr));
             }
             break;
         default:
@@ -452,7 +477,8 @@ static inline TmEcode ProcessErfDagRecords(ErfDagThreadVars *ewtn, uint8_t *top,
  * \param   prec pointer to a DAG record.
  * \param
  */
-static inline TmEcode ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec)
+static inline TmEcode
+ProcessErfDagRecord(ErfDagThreadVars *ewtn, char *prec)
 {
     SCEnter();
 
@@ -541,6 +567,11 @@ ReceiveErfDagThreadExitStats(ThreadVars *tv, void *data)
 {
     ErfDagThreadVars *ewtn = (ErfDagThreadVars *)data;
 
+    (void)SC_ATOMIC_SET(ewtn->livedev->pkts,
+        (uint64_t)SCPerfGetLocalCounterValue(ewtn->packets, tv->sc_perf_pca));
+    (void)SC_ATOMIC_SET(ewtn->livedev->drop,
+        (uint64_t)SCPerfGetLocalCounterValue(ewtn->drops, tv->sc_perf_pca));
+
     SCLogInfo("Stream: %d; Bytes: %"PRIu64"; Packets: %"PRIu64
         "; Drops: %"PRIu64,
         ewtn->dagstream,
@@ -554,7 +585,8 @@ ReceiveErfDagThreadExitStats(ThreadVars *tv, void *data)
  * \param   tv pointer to ThreadVars
  * \param   data pointer that gets cast into PcapThreadVars for ptv
  */
-TmEcode ReceiveErfDagThreadDeinit(ThreadVars *tv, void *data)
+TmEcode
+ReceiveErfDagThreadDeinit(ThreadVars *tv, void *data)
 {
     SCEnter();
 
@@ -565,7 +597,8 @@ TmEcode ReceiveErfDagThreadDeinit(ThreadVars *tv, void *data)
     SCReturnInt(TM_ECODE_OK);
 }
 
-void ReceiveErfDagCloseStream(int dagfd, int stream)
+void
+ReceiveErfDagCloseStream(int dagfd, int stream)
 {
     dag_stop_stream(dagfd, stream);
     dag_detach_stream(dagfd, stream);
@@ -585,15 +618,21 @@ void ReceiveErfDagCloseStream(int dagfd, int stream)
  * \param data pointer that gets cast into PcapThreadVars for ptv
  * \param pq pointer to the current PacketQueue
  */
-TmEcode DecodeErfDag(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
-                   PacketQueue *postpq)
+TmEcode
+DecodeErfDag(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
+    PacketQueue *postpq)
 {
     SCEnter();
     DecodeThreadVars *dtv = (DecodeThreadVars *)data;
 
+    /* XXX HACK: flow timeout can call us for injected pseudo packets
+     *           see bug: https://redmine.openinfosecfoundation.org/issues/1107 */
+    if (p->flags & PKT_PSEUDO_STREAM_END)
+        return TM_ECODE_OK;
+
     /* update counters */
     SCPerfCounterIncr(dtv->counter_pkts, tv->sc_perf_pca);
-    SCPerfCounterIncr(dtv->counter_pkts_per_sec, tv->sc_perf_pca);
+//    SCPerfCounterIncr(dtv->counter_pkts_per_sec, tv->sc_perf_pca);
 
     SCPerfCounterAddUI64(dtv->counter_bytes, tv->sc_perf_pca, GET_PKT_LEN(p));
 #if 0
@@ -602,8 +641,10 @@ TmEcode DecodeErfDag(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
                            (GET_PKT_LEN(p) * 8)/1000000.0);
 #endif
 
-    SCPerfCounterAddUI64(dtv->counter_avg_pkt_size, tv->sc_perf_pca, GET_PKT_LEN(p));
-    SCPerfCounterSetUI64(dtv->counter_max_pkt_size, tv->sc_perf_pca, GET_PKT_LEN(p));
+    SCPerfCounterAddUI64(dtv->counter_avg_pkt_size, tv->sc_perf_pca,
+        GET_PKT_LEN(p));
+    SCPerfCounterSetUI64(dtv->counter_max_pkt_size, tv->sc_perf_pca,
+        GET_PKT_LEN(p));
 
         /* call the decoder */
     switch(p->datalink) {
@@ -612,22 +653,26 @@ TmEcode DecodeErfDag(ThreadVars *tv, Packet *p, void *data, PacketQueue *pq,
             break;
         default:
             SCLogError(SC_ERR_DATALINK_UNIMPLEMENTED,
-                "Error: datalink type %" PRId32 " not yet supported in module DecodeErfDag",
+                "Error: datalink type %" PRId32
+                " not yet supported in module DecodeErfDag",
                 p->datalink);
             break;
     }
 
+    PacketDecodeFinalize(tv, dtv, p);
+
     SCReturnInt(TM_ECODE_OK);
 }
 
-TmEcode DecodeErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
+TmEcode
+DecodeErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
 {
     SCEnter();
     DecodeThreadVars *dtv = NULL;
 
-    dtv = DecodeThreadVarsAlloc();
+    dtv = DecodeThreadVarsAlloc(tv);
 
-    if(dtv == NULL)
+    if (dtv == NULL)
         SCReturnInt(TM_ECODE_FAILED);
 
     DecodeRegisterPerfCounters(dtv, tv);
@@ -637,4 +682,12 @@ TmEcode DecodeErfDagThreadInit(ThreadVars *tv, void *initdata, void **data)
     SCReturnInt(TM_ECODE_OK);
 }
 
+TmEcode
+DecodeErfDagThreadDeinit(ThreadVars *tv, void *data)
+{
+    if (data != NULL)
+        DecodeThreadVarsFree(data);
+    SCReturnInt(TM_ECODE_OK);
+}
+
 #endif /* HAVE_DAG */
